import numpy as np
import matplotlib.pyplot as plt
import csv
from typing import Callable

def _plot_helper(general: np.ndarray, spatial_presence: np.ndarray, 
                 involvement: np.ndarray, realism: np.ndarray) -> tuple:
    # Calculate the average
    general_mean = np.mean(general)
    spatial_presence_mean = np.mean(spatial_presence)
    involvement_mean = np.mean(involvement)
    realism_mean = np.mean(realism)

    # Calculate the standard deviation
    general_std = np.std(general)
    spatial_presence_std = np.std(spatial_presence)
    involvement_std = np.std(involvement)
    realism_std = np.std(realism)

    # Create lists for the plot
    ipq_measurements = ['General', 'Spatial Presence', 'Involvement', 'Realism']
    x_pos = np.arange(len(ipq_measurements))
    CTEs = [general_mean, spatial_presence_mean, involvement_mean, realism_mean]
    error = [general_std, spatial_presence_std, involvement_std, realism_std]

    return (CTEs, error, x_pos, ipq_measurements)

def plot_vr(CTEs: list[float], error: list[float], x_pos: np.ndarray, ipq_measurements: list[str]) -> None:
    # Build the plot
    fig, ax = plt.subplots()
    ax.bar(x_pos, CTEs, yerr=error, align='center', alpha=0.5, ecolor='black', capsize=10)
    ax.set_ylabel('Mean IPQ Score')
    ax.set_xticks(x_pos)
    ax.set_xticklabels(ipq_measurements)
    ax.set_title("VR Demo")
    ax.yaxis.grid(True)

    # Save the figure
    plt.tight_layout()
    plt.savefig('vr_sheet1_exercise2_IPQ_vr_demo.png')
    #plt.show()
    plt.close()

def plot_powerwall(CTEs: list[float], error: list[float], x_pos: np.ndarray, ipq_measurements: list[str]) -> None:
    # Build the plot
    fig, ax = plt.subplots()
    ax.bar(x_pos, CTEs, yerr=error, align='center', alpha=0.5, ecolor='black', capsize=10)
    ax.set_ylabel('Mean IPQ Score')
    ax.set_xticks(x_pos)
    ax.set_xticklabels(ipq_measurements)
    ax.set_title("Powerwall Demo")
    ax.yaxis.grid(True)

    # Save the figure 
    plt.tight_layout()
    plt.savefig('vr_sheet1_exercise2_IPQ_powerwall_demo.png')
    #plt.show()
    plt.close()

def plot_haptics(CTEs: list[float], error: list[float], x_pos: np.ndarray, ipq_measurements: list[str]) -> None:
    # Build the plot
    fig, ax = plt.subplots()
    ax.bar(x_pos, CTEs, yerr=error, align='center', alpha=0.5, ecolor='black', capsize=10)
    ax.set_ylabel('Mean IPQ Score')
    ax.set_xticks(x_pos)
    ax.set_xticklabels(ipq_measurements)
    ax.set_title("Haptics Demo")
    ax.yaxis.grid(True)

    # Save the figure
    plt.tight_layout()
    plt.savefig('vr_sheet1_exercise2_IPQ_haptics_demo.png')
    #plt.show()
    plt.close()

def _read_csv_and_plot(filename: str, plot_f: Callable[..., None]) -> None:
    general = []
    spatial_presence = []
    involvement = []
    realism = []

    # reading csv file
    with open(filename, 'r') as csvfile:
        # creating a csv reader object
        csvreader = csv.reader(csvfile, quoting = csv.QUOTE_NONNUMERIC, skipinitialspace=True)
        
        # step over very first row, which are only the field IDs
        next(csvreader)
    
        # extracting each data row (subject) one by one
        # and averaging spans of values of the same category
        # This is just an example, of course. What you do with the values depends heavily on what you 
        # want to evaluate.
        for row in csvreader:
            general.append(int(row[1]))
            spatial_presence.append(np.average(row[2:6]))
            involvement.append(np.average(row[6:10]))
            realism.append(np.average(row[10:]))

    plot_f(*_plot_helper(np.array(general), np.array(spatial_presence), 
                        np.array(involvement), np.array(realism)))

    

def main() -> None:
    _read_csv_and_plot("ipq_vr_demo.csv", plot_vr)
    _read_csv_and_plot("ipq_powerwall_demo.csv", plot_powerwall)
    _read_csv_and_plot("ipq_haptics_demo.csv", plot_haptics)

if __name__ == '__main__':
    main()